@@ -1,6 +1,7 @@
 package WebService::Pushover;
+use strict;
+
 use Moo;
-no strict 'refs';
 
 binmode STDOUT, ":encoding(UTF-8)";
 
@@ -8,13 +9,13 @@ use Carp;
 use DateTime;
 use DateTime::Format::Strptime;
 use File::Spec;
-use Net::HTTP::Spore;
-use Net::HTTP::Spore::Middleware::Header;
+use WebService::Simple;
+use WebService::Simple::Parser::JSON;
 use Params::Validate qw( :all );
 use Readonly;
 use URI;
 
-use version; our $VERSION = qv('0.2.0');
+use version; our $VERSION = qv('1.0.0');
 
 # Module implementation here
 
@@ -37,22 +38,47 @@ has debug => (
     coerce => sub { $_[0] ? 1 : 0 },
 );
 
-has spore => (
+# NB: We can't call this 'user', as there's already a method called that.
+has user_token => (
+    is       => 'ro',
+    required => 0,
+    isa      => sub { $_[0] =~ /$REGEX_TOKEN/ or die "Invalid user token: $_[0]" },
+);
+
+has api_token => (
+    is       => 'ro',
+    required => 0,
+    isa      => sub { $_[0] =~ /$REGEX_TOKEN/ or die "Invalid api token: $_[0]" },
+);
+
+has base_url => (
+    default => "https://api.pushover.net",
+    is      => 'ro',
+);
+
+has _urls => (
+    is => 'ro',
+    default => sub {
+        return {
+            messages => '/1/messages.json',
+            users    => '/1/users/validate.json',
+            receipts => '/1/receipts/$receipt$.json',
+            sounds   => '/1/sounds.json',
+        };
+    },
+);
+
+has api => (
     is => 'lazy',
 );
 
-sub _build_spore {
-    my $self = shift();
-    my $moddir = $INC{'WebService/Pushover.pm'};
-    my( $volume, $dir, $file ) = File::Spec->splitpath( $moddir );
-    my $spec = File::Spec->catfile( $dir, 'Pushover.json' );
-    ( -r $spec )
-        or croak( "Unable to find SPORE spec: $!" );
-    my $spore = Net::HTTP::Spore->new_from_spec(
-        $spec,
-        trace => $self->debug,
-    ) or croak( "Unable to instantiate SPORE client: $!" );
-    return $spore;
+sub _build_api {
+    my ($self) = @_;
+    return WebService::Simple->new(
+        response_parser => WebService::Simple::Parser::JSON->new,
+        base_url        => $self->base_url,
+        debug           => $self->debug,
+    );
 }
 
 has specs => (
@@ -62,11 +88,6 @@ has specs => (
 sub _build_specs {
     my $self = shift();
     my $SPECS = {
-        format => {
-            type    => SCALAR,
-            regex   => qr/$REGEX_FORMAT/,
-            default => 'json',
-        },
         token => {
             type  => SCALAR,
             regex => qr/$REGEX_TOKEN/,
@@ -182,7 +203,6 @@ sub _build_specs {
     };
 
     my %messages_spec = (
-        format    => $SPECS->{format},
         token     => $SPECS->{token},
         user      => $SPECS->{user},
         device    => $SPECS->{device},
@@ -199,20 +219,17 @@ sub _build_specs {
     );
 
     my %users_spec = (
-        format => $SPECS->{format},
         token  => $SPECS->{token},
         user   => $SPECS->{user},
         device => $SPECS->{device},
     );
 
     my %receipts_spec = (
-        format  => $SPECS->{format},
         token   => $SPECS->{token},
         receipt => $SPECS->{receipt},
     );
 
     my %sounds_spec = (
-        format => $SPECS->{format},
         token  => $SPECS->{token},
     );
 
@@ -225,51 +242,59 @@ sub _build_specs {
 }
 
 sub _apicall {
-    my $self = shift;
-
-    my $call = shift;
+    my ($self, $method, $call, @rest) = @_;
 
     my $spec = $self->specs->{$call}
         or croak( "'$call' is not a supported API call." );
+    my $url    = $self->_urls->{$call}
+         or croak( "'$call' is not a supported API call." );
+    my $params = validate( @rest, $spec );
 
-    my $params = validate( @_, $spec );
-
-    my $response;
-    if ( defined( $params->{format} ) && $params->{format} eq 'json' ) {
-        $response = $self->spore->enable( 'Header', header_name => 'Content-Type', header_value => 'application/x-www-form-urlencoded' )->enable( 'Format::JSON' )->$call( %{$params} );
-    }
-    elsif ( defined( $params->{format} ) && $params->{format} eq 'xml' ){
-        $response = $self->spore->enable( 'Header', header_name => 'Content-Type', header_value => 'application/x-www-form-urlencoded' )->enable( 'Format::XML' )->$call( %{$params} );
-    }
-    else {
-        $response = $self->spore->$call( %{$params} );
+    while ($url =~ /\$(\S+?)\$/) {
+        my $arg = $1;
+        my $val = delete($params->{$arg}) || "";
+        $url =~ s/\$$arg\$/$val/g;
     }
 
-    return $response->body;
+    return $self->api->$method($url, $params)->parse_response;
 }
 
 sub message {
-    my $self = shift;
+    my ($self, %opts) = @_;
 
-    return $self->_apicall( 'messages', @_ );
+    return $self->_apicall(post => 'messages',
+        user  => $self->user_token,
+        token => $self->api_token,
+        %opts,
+    );
 }
 
 sub user {
-    my $self = shift;
+    my ($self, %opts) = @_;
 
-    return $self->_apicall( 'users', @_ );
+    return $self->_apicall(post => 'users',
+        user  => $self->user_token,
+        token => $self->api_token,
+        %opts,
+    );
 }
 
 sub receipt {
-    my $self = shift;
+    my ($self, %opts) = @_;
 
-    return $self->_apicall( 'receipts', @_ );
+    return $self->_apicall(get => 'receipts',
+        token => $self->api_token,
+        %opts,
+    );
 }
 
 sub sounds {
-    my $self = shift;
+    my ($self, %opts) = @_;
 
-    return $self->_apicall( 'sounds', @_ );
+    return $self->_apicall(get => 'sounds',
+        token => $self->api_token,
+        %opts,
+    );
 }
 
 # ok, add some backwards compatibility
@@ -298,23 +323,22 @@ __END__
 
 WebService::Pushover - interface to Pushover API
 
-
 =head1 VERSION
 
-This document describes WebService::Pushover version 0.2.0.
-
+This document describes WebService::Pushover version 1.0.0.
 
 =head1 SYNOPSIS
 
     use WebService::Pushover;
 
-    my $push = WebService::Pushover->new
-        or die( "Unable to instantiate WebService::Pushover.\n" );
+    my $push = WebService::Pushover->new(
+        user_token => 'PUSHOVER USER TOKEN',
+        api_token  => 'PUSHOVER API TOKEN',
+    ) or die( "Unable to instantiate WebService::Pushover.\n" );
 
     my %params = (
-        token => 'PUSHOVER API TOKEN',
-        user => 'PUSHOVER USER TOKEN',
-        message => 'test test test',
+        message  => 'test test test',
+        priority => 0,
     );
 
     my $status = $push->message( %params );
@@ -330,10 +354,32 @@ before you'll be able to do anything with this module.
 
 =over 4
 
-=item new()
+=item new(I<%params>)
 
-Accepts a single parameter, C<debug>.  Set this to a true value in order to
-enable tracing of L<Net::HTTP::Spore> operations.
+Creates an object that allows for interaction with Pushover. The following
+are valid arguments; all are optional:
+
+=over 4
+
+=item base_url B<OPTIONAL>
+
+Sets the base URL for the API to connect to. Defaults to L<http://api.pushover.net>
+
+=item user_token B<OPTIONAL>
+
+The Pushover user token, obtained by registering at L<http://pushover.net>.
+If specified, will be used as a default in any call that requires a user token.
+
+=item api_token B<OPTIONAL>
+
+The Pushover application token, obtained by registering at L<http://pushover.net/apps>.
+If specified, will be used as a default in any call that requires an API token.
+
+=item debug B<OPTIONAL>
+
+Set this to a true value in order to enable tracing of API call operations.
+
+=back
 
 =item debug()
 
@@ -346,13 +392,15 @@ representation of the message status.  The following are valid parameters:
 
 =over 4
 
-=item token B<REQUIRED>
+=item token B<OPTIONAL>
 
-The Pushover application token, obtained by registering at L<http://pushover.net/apps>.
+The Pushover application token.
+If not specified, the C<api_token> specified in C<new> will be used.
 
-=item user B<REQUIRED>
+=item user B<OPTIONAL>
 
-The Pushover user token, obtained by registering at L<http://pushover.net>.
+The Pushover user token.
+If not specified, the C<user_token> specified in C<new> will be used.
 
 =item device B<OPTIONAL>
 
@@ -405,10 +453,6 @@ documentation for valid values.
 
 =back
 
-=item push(I<%params>)
-
-I<push()> is a B<DEPRECATED> alias for I<message()>.  It will be removed in a future release, but remains for backwards compatibility.
-
 =item user(I<%params>)
 
 I<user()> sends an application token and a user token to Pushover and
@@ -417,13 +461,15 @@ following are valid parameters:
 
 =over 4
 
-=item token B<REQUIRED>
+=item token B<OPTIONAL>
 
-The Pushover application token, obtained by registering at L<http://pushover.net/apps>.
+The Pushover application token.
+If not specified, the C<api_token> specified in C<new> will be used.
 
-=item user B<REQUIRED>
+=item user B<OPTIONAL>
 
-The Pushover user token, obtained by registering at L<http://pushover.net>.
+The Pushover user token.
+If not specified, the C<user_token> specified in C<new> will be used.
 
 =item device B<OPTIONAL>
 
@@ -432,10 +478,6 @@ registered to the user token.
 
 =back
 
-=item tokens(I<%params>)
-
-I<tokens()> is a B<DEPRECATED> alias for I<user()>.  It will be removed in a future release, but remains for backwards compatibility.
-
 =item receipt(I<%params>)
 
 I<receipt()> sends an application token and a receipt token to Pushover and
@@ -444,9 +486,10 @@ notification associated with the receipt.  The following are valid parameters:
 
 =over 4
 
-=item token B<REQUIRED>
+=item token B<OPTIONAL>
 
-The Pushover application token, obtained by registering at L<http://pushover.net/apps>.
+The Pushover application token.
+If not specified, the C<api_token> specified in C<new> will be used.
 
 =item receipt B<REQUIRED>
 
@@ -463,14 +506,32 @@ the I<message()>.  The following are valid parameters:
 
 =over 4
 
-=item token B<REQUIRED>
+=item token B<OPTIONAL>
 
-The Pushover application token, obtained by registering at L<http://pushover.net/apps>.
+The Pushover application token.
+If not specified, the C<api_token> specified in C<new> will be used.
 
 =back
 
 =back
 
+=head2 DEPRECATED METHODS
+
+The following methods are depreated, and will be removed in a future
+release. They remain for now to provide backwards compatibility.
+
+=over
+
+=item push(I<%params>)
+
+I<push()> is a B<DEPRECATED> alias for I<message()>.
+
+=item tokens(I<%params>)
+
+I<tokens()> is a B<DEPRECATED> alias for I<user()>.
+
+=back
+
 =head1 DIAGNOSTICS
 
 Inspect the value returned by any method, which will be a Perl data structure
@@ -486,9 +547,9 @@ parsed from the JSON or XML response returned by the Pushover API.
 
 =item L<Moo>
 
-=item L<Net::HTTP::Spore>
+=item L<WebService::Simple>
 
-=item L<Net::HTTP::Spore::Middleware::Header>
+=item L<WebService::Simple::Parser::JSON>
 
 =item L<Params::Validate>
 
@@ -502,9 +563,7 @@ parsed from the JSON or XML response returned by the Pushover API.
 
 No bugs have been reported.
 
-Please report any bugs or feature requests to
-C<bug-webservice-pushover@rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org>.
+Please report any bugs or feature requests through the web interface at L<https://github.com/hakamadare/webservice-pushover/issues>.
 
 =head1 AUTHOR
 
